<?php
class fostarAPI {
  private function __construct() {}
  
  private function __clone() {}

  static private function apiGetContentCurl($url) {
  $ch = curl_init();

  curl_setopt ($ch, CURLOPT_URL, $url);
  curl_setopt ($ch, CURLOPT_HEADER, 0);

  ob_start();

  curl_exec ($ch);
  curl_close ($ch);
  $string = ob_get_contents();

  ob_end_clean();

  return $string;
}
  
  static private function getResult($url) {
    $res = cache_get("fostarAPI:$url", 'cache');
    if ($res === 0) {
      if (ini_get('allow_url_fopen')===1) {
          $res = file_get_contents(variable_get('fostar_api_url', 'http://api2.fostar.ru/interface.php?') . $url,FALSE);
      } else {
          $res = self::apiGetContentCurl(variable_get('fostar_api_url', 'http://api2.fostar.ru/interface.php?') . $url,FALSE);
      }
      
      $res = iconv('cp1251', mb_internal_encoding(), $res);

      if($res == 'false') {
        return 'false';
      }

      cache_set("fostarAPI:$url", $res, 'cache', CACHE_PERMANENT);
    } else {
      $res = $res->data;
    }
    
    return $res;
  }

  static private function makeArrayFromResult($res) {
    $rez = explode("\r\n", $res);
    return $rez;
  }

  static private function makeMainArrayFromResult($res) {
    $rez = explode(":::", $res);
    return $rez;
  }
  
  static private function fetchImage($image_source_name, $product_id) {
    $tmp_path = file_directory_temp();
    $images_path = $_SERVER['DOCUMENT_ROOT'] . base_path(). file_directory_path() . '/' . variable_get('fostar_images_folder', 'fostar-images');
    
    $tmp_file = $tmp_path . "/$image_source_name";
    $target_file = $images_path . "/$image_source_name";
    
    
    if (!file_exists($target_file)) {
      $image_source = str_replace(" ", "%20", "http://img.fostar.ru/$image_source_name");
      $image = @file_get_contents($image_source);
      if ($image) {
        file_save_data($image, $tmp_file);
        file_move($tmp_file, $target_file, FILE_EXISTS_ERROR);
      }
    }
    return $image_source_name;
  }
  
  static public function updatePartnerInfo() {
    global $base_url;    
    
    $partner_id = variable_get('fostar_partner_id', '0');
    $version = VERSION;
    
    $partner_url = base64_encode($base_url);
    $partner_clear_cache_url = base64_encode("$base_url/fostar/clear-cache");
    $partner_cms = base64_encode("Drupal $version");
    self::getResult("act=update_partner_info&partner_id=$partner_id&partner_cms=$partner_cms&partner_url=$partner_url&partner_clear_cache_url=$partner_clear_cache_url");
    
    $user_agent = base64_encode($_SERVER['HTTP_USER_AGENT']);
    self::getResult("act=visitor_info&partner_id=$partner_id&REMOTE_ADDR={$_SERVER['REMOTE_ADDR']}&HTTP_X_FORWARDED_FOR={$_SERVER['HTTP_X_FORWARDED_FOR']}&HTTP_X_REAL_IP={$_SERVER['HTTP_X_REAL_IP']}&HTTP_USER_AGENT=$user_agent");
  }

  static public function getCategoriesTree() {
    $res = cache_get("fostarAPI:CategoriesTree", 'cache');
    
    if ($res === 0) {
      $tree = array();
      $res = self::getResult('act=parents');
      if($res != 'false') {
        $res = self::makeArrayFromResult($res);

        foreach ($res as &$r) {
          $item = self::makeMainArrayFromResult($r);
          $tree[$item[0]] = self::getCategory($item[0]);
         
          
        }

        foreach ($tree as $category_id => &$t) {
          $res = self::getResult("act=childs&CID=$category_id");
          $res = self::makeArrayFromResult($res);
          foreach ($res as &$r) {
            $item = self::makeMainArrayFromResult($r);
            $t['childs'][$item[0]] = self::getCategory($item[0]);
          }

        }
        
        cache_set("fostarAPI:CategoriesTree", $tree, 'cache', CACHE_PERMANENT);
      }
    } else {
      $tree = $res->data;
    }
    
    return $tree;
  }
  
  static public function getCategory($category_id) {
    $res = cache_get("fostarAPI:Category:$category_id", 'cache');
    
    if ($res === 0) {
      $tree = array();
      $res = self::getResult("act=categories&CID=$category_id");
      if($res != 'false') {
        $res = self::makeMainArrayFromResult($res);
        
        if ($res[2]) {
          $parent_category = self::getCategory($res[2]);
          $category_full_id = "{$parent_category['category_full_id']}/{$res[0]}";
          $category_full_name = "{$parent_category['category_full_name']}/{$res[1]}";
        } else {
          $category_full_id = $res[0];
          $category_full_name = $res[1];
        }
        
        $category = array(
          'category_id' => $res[0],
          'category_name' => $res[1],
          'category_parent_id' => $res[2],
          'category_description' => self::getResult("act=cat_description&CID=$category_id&partner_id=" . variable_get('fostar_partner_id', '0')),
          'category_full_id' => $category_full_id,
          'category_full_name' => $category_full_name,
        );
        
        fostar_category_update_alias((object) $category);
        cache_set("fostarAPI:Category:$category_id", $category, 'cache', CACHE_PERMANENT);
      }
    } else {
      $category = $res->data;
    }
    
    return $category;
  }
  
  static public function getProducts($category_id, $offset = 0, $limit = 0) {
    $res = cache_get("fostarAPI:Products:$category_id:$offset:$limit", 'cache');
    if ($res === 0) {
      $products = array();

      if ($category_id) {
        $category = self::getCategory($category_id);
        if ($category['category_parent_id']) {
          $res = self::getResult("act=goods&CID=$category_id&offset=$offset&count=$limit&partner_id=" . variable_get('fostar_partner_id', 0));
        } else {
          $categories_tree = self::getCategoriesTree();    
          $child_categories = array();
          
          foreach ($categories_tree[$category_id]['childs'] as &$c) $child_categories[] = $c['category_id'];
          $child_categories = implode(':::', $child_categories);
          
          $res = self::getResult("act=latest&cids=$child_categories&partner_id=" . variable_get('fostar_partner_id', 0) . "&offset=$offset&count=$limit");
        }
      } else {
        return false;
      }
      
      if($res != 'false') {
        $res = self::makeArrayFromResult($res);

        foreach ($res as &$r) {
          $item = self::makeMainArrayFromResult($r);
          $products[$item[0]] = self::getProduct($item[0]);
        }

        cache_set("fostarAPI:Products:$category_id:$offset:$limit", $products, 'cache', CACHE_PERMANENT);
      }
    } else {
      $products = $res->data;
    }
    
    return $products;
  }

  static public function getProductsCount($category_id) {
    $res = cache_get("fostarAPI:ProductsCount:$category_id", 'cache');
    if ($res === 0) {
      if ($category_id) {
        $category = self::getCategory($category_id);
        if ($category['category_parent_id']) {
          $res = self::getResult("act=products_count&cids=$category_id");
        } else {
          $categories_tree = self::getCategoriesTree();    
          $child_categories = array();
          
          foreach ($categories_tree[$category_id]['childs'] as &$c) $child_categories[] = $c['category_id'];
          $child_categories = implode(':::', $child_categories);
          
          $res = self::getResult("act=products_count&cids=$child_categories");
        }
      } else {
        return false;
      }
      
      if($res != 'false') {
        cache_set("fostarAPI:ProductsCount:$category_id", $res, 'cache', CACHE_PERMANENT);
      }
    } else {
      $res = $res->data;
    }
    
    return $res;
  }
  
  static public function getProductRelations($product_id) {
    $res = cache_get("fostarAPI:ProductRelations:$product_id", 'cache');
    
    if ($res === 0) {
      $res = self::getResult("act=relations&GID=$product_id");
      if ($res == 'false') return false;

      $relations = array();
      $items = self::makeArrayFromResult($res);
      foreach ($items as &$i) {
        $item = self::makeMainArrayFromResult($i);
        $relations[] = $item[0];
      }
      
      cache_set("fostarAPI:ProductRelations:$product_id", $relations, 'cache', CACHE_PERMANENT);
    } else {
        $relations = $res->data;
    }
    
    return $relations;
  }

  static public function getProduct($product_id) {
    $res = cache_get("fostarAPI:Product:$product_id", 'cache');
    
    if ($res === 0) {
      $res = self::getResult("act=good&GID=$product_id&partner_id=" . variable_get('fostar_partner_id', 0));
      if ($res == 'false') return false;
      
      $item = self::makeMainArrayFromResult($res);
      
      $product_category = self::getCategory($item[5]);

      $product = array(
        'product_id' => $product_id,
        'product_name' => $item[1],
        'image' => $item[3],
        'special' => $item[4],
        'category_id' => $item[5],
        'category_full_id' => $product_category['category_full_id'],
        'category_full_name' => $product_category['category_full_name'],
        'price' => $item[6],
        'category_name' => $item[7],
        'product_description' => $item[8],
        'on_order_only' => $item[9],
        'product_sku' => $item[12],
        'price_increment' => $item[13],
        'image' => self::fetchImage($item[3], $product_id),
        'relations' => self::getProductRelations($product_id),
      );
      
      $product['additional_images'] = array();
      
      $res = self::getResult("act=photo&GID=$product_id");
      if ($res !== 'false') {
        $res = self::makeArrayFromResult($res);
        foreach ($res as &$r) {
          $item = self::makeMainArrayFromResult($r);
          $product['additional_images'][] = self::fetchImage(str_replace('http://fostar.ru/components/com_virtuemart/shop_image/product/', '', $item[2]), $product_id);
        }
        
      }
      
      fostar_product_update_alias((object) $product, 'update');
      cache_set("fostarAPI:Product:$product_id", $product, 'cache', CACHE_PERMANENT);
    } else {
        $product = $res->data;
    }
    
    return $product;
  }
  
  static public function getContent($content_id) {
    $res = cache_get("fostarAPI:Content:$content_id", 'cache');
    
    if ($res === 0) {
      $res = self::getResult("act=content&PID=$content_id");
      if ($res == 'false') return false;
      
      $item = self::makeMainArrayFromResult($res);

      $content = array(
        'content_id' => $item[0],
        'title' => $item[1],
        'text' => $item[2],
      );
      
      cache_set("fostarAPI:Content:$content_id", $content, 'cache', CACHE_PERMANENT);
    } else {
        $content = $res->data;
    }
    
    return $content;
  }
  
  static public function getSeasonAction() {
    $res = cache_get("fostarAPI:SeasonAction", 'cache');
    
    if ($res === 0) {
      $res = self::getResult("act=season_action");
      if ($res == 'false') return false;
      
      $item = self::makeMainArrayFromResult($res);

      $season_action = array(
        'content_id' => $item[0],
        'content' => self::getContent($item[0]),
        'link_text' => $item[1],
      );
      
      cache_set("fostarAPI:SeasonAction", $season_action, 'cache', CACHE_PERMANENT);
    } else {
        $season_action = $res->data;
    }
    
    return $season_action;
  }
  
  static public function getShippingMethods() {
    return array(
      '1' => array(
        'id' => 1,
        'name' => 'Курьером по Москве',
        'price' => 150,
      ),
      '2' => array(
        'id' => 2,
        'name' => 'Экспресс доставка (Почта России)',
        'price' => 300,
      ),
      '3' => array(
        'id' => 3,
        'name' => 'Экспресс доставка (DHL)',
        'price' => 2000,
      ),
    );
  }
  
  static public function getPaymentMethods() {
    return array(
      '2' => array(
        'id' => 2,
        'name' => 'Оплата при получении',
      ),
      '25' => array(
        'id' => 25,
        'name' => 'Квитанция сбербанк',
      ),
    );
  }
  
  static public function submitOrder($order) {
    $ord = strtr(base64_encode(addslashes(gzcompress(serialize($order),9))), '+/=', '-_,');
    $res = self::getResult("act=register&ord=$ord&partner_id=" . variable_get('fostar_partner_id', 1005));
    if ($res == 'false') {
      return false;
    } else {
      return true;
    }
  }
  
}
