<?php
function fostar_page_category($category) {
  $parent_category = false;
  $child_categories_links = array();
  
  if ($category['category_parent_id']) {
    $parent_category = fostarAPI::getCategory($category['category_parent_id']);
    drupal_set_title("{$parent_category['category_name']} &mdash; {$category['category_name']}", PASS_THROUGH);
    drupal_set_breadcrumb(array(
      l("Каталог", variable_get('site_frontpage', 'fostar-catalog') == "fostar-catalog" ? '<front>' : 'fostar-catalog'),
      l($parent_category['category_name'], "fostar/category/{$parent_category['category_id']}"),
    ));
  } else {
    drupal_set_title($category['category_name']);
    drupal_set_breadcrumb(array(
      l("Каталог", variable_get('site_frontpage', 'fostar-catalog') == "fostar-catalog" ? '<front>' : 'fostar-catalog'),
    ));
    
    $categories_tree = fostarAPI::getCategoriesTree();    
    foreach ($categories_tree[$category['category_id']]['childs'] as &$c) {
      $child_categories_links[] = array(
        'href' => "fostar/category/{$c['category_id']}",
        'title' => $c['category_name'],
      );
    }
  }
  
  $output = "";
  $output .= theme('links', $child_categories_links);
  $output .= "<p>{$category['category_description']}</p>";
  
  $products_count = fostarAPI::getProductsCount($category['category_id']);
  $page = isset($_GET['page']) ? intval($_GET['page']) : 0;
  $num_per_page = variable_get('fostar_products_per_page', 30);
  $offset = $num_per_page * $page;
  $products = fostarAPI::getProducts($category['category_id'], $offset, $num_per_page);
  $images_path = file_directory_path() . '/' . variable_get('fostar_images_folder', 'fostar-images'). '/';
  foreach ($products as &$p) {
    $product = fostar_product_load(array($p['product_id']));
    
    if ($p['image']) {
      $preset = imagecache_preset(variable_get('fostar_lists_image_style', ''));
      $image_style_url = imagecache_create_url($preset['presetname'], $images_path . $p['image'], false, false);
      $p['img'] = "<img src=\"$image_style_url\" />";
    } else {
      $p['img'] = '';
    }
    
    $output .= theme('fostar_product_teaser', $p);
  }

  global $pager_page_array, $pager_total, $pager_total_items;
  
  $page = isset($_GET['page']) ? $_GET['page'] : '';
  $pager_page_array = explode(',', $page);
  
  $pager_total_items[0] = $products_count;
  $pager_total[0] = ceil($pager_total_items[0] / $num_per_page);
  $pager_page_array[0] = max(0, min((int)$pager_page_array[0], ((int)$pager_total[0]) - 1));
  
  $output .= theme('pager', NULL, $num_per_page);
  
  return $output;
}

function fostar_page_product($product) {
  drupal_set_title($product['product_name']);
  $breadcrumb = array(
    l("Каталог", variable_get('site_frontpage', '') == "fostar-catalog" ? '<front>' : 'fostar-catalog'),
  );
  
  $category = fostarAPI::getCategory($product['category_id']);
  if ($category['category_parent_id']) {
    $parent_category = fostarAPI::getCategory($category['category_parent_id']);
    $breadcrumb[] = l($parent_category['category_name'], "fostar/category/{$parent_category['category_id']}");
  }
  $breadcrumb[] = l($category['category_name'], "fostar/category/{$category['category_id']}");
  drupal_set_breadcrumb($breadcrumb);
  
  $images_path = file_directory_path() . '/' . variable_get('fostar_images_folder', 'fostar-images'). '/';
  if ($product['image']) {
    $preset = imagecache_preset(variable_get('fostar_main_image_style', ''));
    $image_style_url = imagecache_create_url($preset['presetname'], $images_path . $product['image'], false, false);
    $product['main_image']['preview'] = "<img src=\"$image_style_url\" />";
    $product['main_image']['file'] = base_path() . $images_path . $product['image'];
  } else {
    $product['main_image'] = '';
  }
  
  $additional_images = array();
  foreach ($product['additional_images'] as $image) {
    if ($image) {
      $preset = imagecache_preset(variable_get('fostar_additional_image_style', ''));
      $image_style_url = imagecache_create_url($preset['presetname'], $images_path . $image, false, false);
      $additional_images[] = array(
        'preview' => "<img src=\"$image_style_url\" />",
        'file' => base_path() . $images_path . $image,
      );
    }
  }
  $product['additional_images'] = $additional_images;
  
  $product['season_action'] = fostarAPI::getSeasonAction();
  $product['related_products'] = array();
  if ($product['relations']) {
    foreach ($product['relations'] as $r) {
      $p = fostar_product_load($r);
      if ($p['image']) {
        $preset = imagecache_preset(variable_get('fostar_lists_image_style', ''));
        $image_style_url = imagecache_create_url($preset['presetname'], $images_path . $p['image'], false, false);
        $p['img'] = "<img src=\"$image_style_url\" />";
      } else {
        $p['img'] = '';
      }
      $product['related_products'][] = $p;
    }
  }
  
  return theme('fostar_product', $product);
}

function fostar_page_index() {
  $categories_tree = fostarAPI::getCategoriesTree();    
  foreach ($categories_tree as &$cp) {
    $child_categories_links = array();
    foreach ($cp['childs'] as &$c) {
      $child_categories_links[] = array(
        'href' => "fostar/category/{$c['category_id']}",
        'title' => $c['category_name'],
      );

      $cp['child_categories_links'] = theme('links', $child_categories_links);
    }
  }
  
  $page_array['categories_tree'] = array(
    '#theme' => 'fostar_index',
    '#categories_tree' => $categories_tree,
  );

  return theme('fostar_index', $categories_tree);
}

function fostar_page_cart() {
  drupal_set_title('Ваша корзина');
  drupal_set_breadcrumb(array(
    l("Каталог", variable_get('site_frontpage', 'fostar-catalog') == "fostar-catalog" ? '<front>' : 'fostar-catalog'),
  ));
  
  if (!count($_SESSION['fostar']['cart'])) {
    return array(
      'notify' => array(
        '#type' => 'markup',
        '#value' => '<p>Ваша корзина пуста.</p>'
      ),
    );
  }
  
  foreach ($_SESSION['fostar']['cart'] as &$item) {
    $form['quantity'][$item['product_id']] = array(
      '#parents' => array('quantity', $item['product_id']),
      '#id' => "quantity-{$item['product_id']}",
      '#type' => 'textfield',
      '#default_value' => $item['quantity'],
      '#size' => 1,
    );
  }

  $form['refresh'] = array(
    '#type' => 'submit',
    '#value' => 'Пересчитать',
    '#submit' => array('fostar_page_cart_refresh'),
  );
  
  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => 'Все верно, оформить заказ',
    '#submit' => array('fostar_page_cart_submit'),
  );

  $form['#theme'] = 'fostar_cart';
  
  return $form;
}

function fostar_page_cart_validate($form, &$form_state) {
  foreach ($form_state['values']['quantity'] as $i => &$item) {
    if (!is_numeric($item['quantity'])) {
      form_set_error("quantity][$i", 'Неверное значение');
    }    
  }
}

function fostar_page_cart_refresh($form, &$form_state) {
  foreach ($form_state['values']['quantity'] as $product_id => $quantity) {
    if (!$quantity) {
      unset($_SESSION['fostar']['cart'][$product_id]);
    } else {
      $_SESSION['fostar']['cart'][$product_id]['quantity'] = $quantity;
    }
  }
}

function fostar_page_cart_submit($form, &$form_state) {
  foreach ($form_state['values']['quantity'] as $product_id => $quantity) {
    if (!$quantity) {
      unset($_SESSION['fostar']['cart'][$product_id]);
    } else {
      $_SESSION['fostar']['cart'][$product_id]['quantity'] = $quantity;
    }
  }
  drupal_goto('fostar/checkout');
}

function fostar_page_checkout() {
  if (!count($_SESSION['fostar']['cart'])) drupal_goto('fostar/cart');
  
  drupal_set_title('Оформление заказа');
  drupal_set_breadcrumb(array(
    l("Каталог", variable_get('site_frontpage', 'fostar-catalog') == "fostar-catalog" ? '<front>' : 'fostar-catalog'),
    l("Ваша корзина", 'fostar/cart'),
  ));
  
  $form['description'] = array(
    '#type' => 'item',
    '#markup' => '<p>Поля, отмеченные звёздочкой, обязательны для заполнения</p>',
  );
  
  $form['last_name'] = array(
    '#type' => 'textfield',
    '#title' => 'Фамилия',
    '#required' => true,
  );
  
  $form['first_name'] = array(
    '#type' => 'textfield',
    '#title' => 'Имя',
    '#required' => true,
  );
  
  $form['middle_name'] = array(
    '#type' => 'textfield',
    '#title' => 'Отчество',
  );
  
  $form['address'] = array(
    '#type' => 'textfield',
    '#title' => 'Адрес',
    '#required' => true,
  );
  
  $form['phone'] = array(
    '#type' => 'textfield',
    '#title' => 'Контактный телефон',
    '#required' => true,
  );
  
  $form['email'] = array(
    '#type' => 'textfield',
    '#title' => 'E-mail',
    '#required' => true,
  );
  
  $form['shipping_method'] = array(
    '#type' => 'select',
    '#title' => 'Способ доставки',
    '#options' => array(),
    '#required' => true,
  );
  
  foreach (fostarAPI::getShippingMethods() as $sm) $form['shipping_method']['#options'][$sm['id']] = "{$sm['name']} (" . number_format($sm['price'], 0, '.', ' ') . " руб.)";

  $form['payment_method'] = array(
    '#type' => 'select',
    '#title' => 'Способ оплаты',
    '#options' => array(),
    '#required' => true,    
  );
  
  foreach (fostarAPI::getPaymentMethods() as $pm) $form['payment_method']['#options'][$pm['id']] = $pm['name'];

  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => 'Все верно, оформить заказ',
  );
  
  $form['#theme'] = 'fostar_checkout';
  return $form;
}

function fostar_page_checkout_validate($form, &$form_state) {
  if (!valid_email_address($form_state['values']['email'])) form_set_error('email', 'Укажите корректный адрес E-mail');
}

function fostar_page_checkout_submit($form, &$form_state) {
  $order = array(
    'cDeliveryMethod' => $form_state['values']['shipping_method'],
    'cPaymentMethod' => $form_state['values']['payment_method'],
    'cFam' => $form_state['values']['last_name'],
    'cName' => $form_state['values']['first_name'],
    'oOtchestv' => $form_state['values']['middle_name'],
    'cPhone' => $form_state['values']['phone'],
    'oAddress' => $form_state['values']['address'],
    'cEmail' => $form_state['values']['email'],
  );

  foreach ($order as &$o) $o = iconv("utf-8","windows-1251", $o);
  $order['productList'] = array();
  
  foreach ($_SESSION['fostar']['cart'] as &$item) $order['productList'][$item['product_id']] = $item['quantity'];
  
  fostarAPI::submitOrder($order);
  unset($_SESSION['fostar']['cart']);
  drupal_goto('fostar/checkout/complete');
}

function fostar_page_checkout_complete() {
  drupal_set_title('Ваш заказ отправлен');  
  drupal_set_breadcrumb(array(
    l("Каталог", variable_get('site_frontpage', 'fostar-catalog') == "fostar-catalog" ? '<front>' : 'fostar-catalog'),
  ));
  
  $output = '<p>Спасибо за заказ, в ближайшее время наши менеджеры свяжутся с Вами.</p>';
  return $output;
}
